// Agent.java

import java.awt.*;
import java.util.LinkedList;

public class Agent {
  // data
  public double x;
  public double y;
  public double vx;
  public double vy;
  public Color color;
  public int radius;
  public double scale;
  protected int nature;  // type of object:
  /*                            0 - generic agent
   *                            1 - deer
   *                            2 - wolf
   *                            3 - something else
   */
  protected static final int AGENT = 0;
  protected static final int DEER = 1;
  protected static final int WOLF = 2;
  protected static final int SOMETHINGELSE = 3;
  
  protected int targetX;
  protected int targetY;
  protected boolean foundOne;
  protected Agent target;

  protected int state;
  protected static final int NORMAL = 0;
  protected static final int HUNGRY = 1;
  protected static final int DEAD = 2;
  protected static final int SCARED = 3;
  
  
  public Agent() {
    Dimension screen = Global.screenSize;
    vx = 0;
    vy = 0;
    x = screen.width * Math.random();
    y = screen.height * Math.random();
    color = Color.BLACK;
    radius = 5;
    scale = 5;
    nature = AGENT;
    state = NORMAL;
  }
  
  public void update(LinkedList<Agent> lst) {
    // Brownian Motion
    vx += scale * (Math.random() - 0.5);
    vy += scale * (Math.random() - 0.5);
    
    // Position update
    x += (int)(vx * Global.DT);
    y += (int)(vy * Global.DT);
    
    // wall bounce logic
    if(x < radius) {
      vx = (int) Math.abs(vx) + 1;
      x = radius;
    }
    if(y < radius) {
      vy = (int) Math.abs(vy) + 1;
      y = radius;
    }
    Dimension screen = Global.screenSize;
    if(x > (screen.width - radius)) {
      vx = - (int) Math.abs(vx) - 1;
      x = screen.width - radius;
    }
    if(y > (screen.height - radius)) {
      vy = - (int) Math.abs(vy) - 1;
      y = screen.height - radius;
    }
  }
  
  public void draw(Graphics g) {
    g.setColor(color);
    g.fillOval( (int) x - radius, (int) y - radius, 2*radius, 2*radius);
  }
  
  /*
   *  look for the nearest Agent of the given type
   *  @param lst the list of agents
   *  @param type the type of agent
   *  @return nothing - results in targetX, targetY, foundOne, target
   */
  
  protected void nearestThing(LinkedList<Agent> lst, int type, int dist) {
    
    target = null;
    foundOne = false;
    // go down the list
    for(int i = 0; i < lst.size(); i++) {
      Agent it = lst.get(i);
      // is it the nature you want and not dead?
      if( it.nature == type && it.state != DEAD  ) {     
      // if so, how close is it?
        double dx = x - it.x;
        double dy = y - it.y;
        int itsDist = (int) Math.sqrt( dx*dx + dy*dy);
      
      // if close than dist, save its location
        if(itsDist < dist) {
           foundOne = true;
           targetX = (int) it.x;
           targetY = (int) it.y;
           dist = itsDist;
           target = it;
        }     
      }
    }
  }
}