/**
 * A class providing the storage for the simulation
 */

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.event.*;
import java.io.*;
import javax.swing.JOptionPane;
import javax.swing.JFileChooser;
import java.util.Date;
import java.util.StringTokenizer;
import java.util.LinkedList;

/**
 * an essentially static class containing the simulation
 */
public class Model {
    
   /**
   *  MODEL Section
   */
    
    /** Title of the main window  **/
    public static String title = "Wolf and Deer Simulation";
    /** Labels on the custom buttons  **/
    public static String Labels[] = { "Reset","Cry Wolf"};
    
    /** repainting the screen */
    public boolean doRepaint;
    /** solution is running */
    public boolean isRunning;
    /** painting the screen to protect the data while painting */
    private boolean painting;
    /** system initialized? */
    private boolean initialized;
    /* time */
    private int t;
    
    private Color blink;
    private boolean isBlack;
    
    private LinkedList<Agent> list;
    
    
    /**
     * basic constructor
     */
    public Model() {
        Global.doDbg = false;
        reset();
    }
    
    private void setSize(Dimension d) {
        Global.screenSize = d;
    }
    
    
    private void reset() {
        t = 0;
        Global.DT = 0.0333333333;
        initialized = false;
        isBlack = true;
        blink = Color.BLACK;
        list = new LinkedList<Agent>();
    }
    
    
    /**
     * hit here each clock tick
     */
    public void timerTick() {
        t++;
        
        // dynamics of the pendulum
        
        if(t%30 == 0) {
          if(isBlack) {
            blink = Color.WHITE;
          } else {
            blink = Color.BLACK;
          }
          isBlack = !isBlack;
        }
        
        for(int i = 0; i < list.size(); i++) {
           Agent it = list.get(i);
           it.update(list);
        }
        if(!painting) doRepaint = true;
    }
    
    
    /**
     *  VIEW Section
     */
    
    private Color backColor = new Color(150, 255, 150);
    
    
    private void initializeDisplay(Dimension d) {
        Global.screenSize = d;
        initialized = true;
        for(int i = 0; i < 13; i++) {
          list.add(new Deer());
        }
    }
    /**
     * paint the screen
     * @param g the graphic environment
     * @param d the screen size
     */
    public void doPaint(Graphics g, Dimension d) {
        
        setSize(d);
        if(!initialized) initializeDisplay(d);
        painting = true;
        g.setColor(backColor);
        g.fillRect(0, 0, d.width, d.height);
        // Do all drawing here
//        g.setColor(blink);
//        g.fillRect(10, 10, 20, 20);
        
        // draw your stuff
        for(int i = 0; i < list.size(); i++) {
           Agent it = list.get(i);
           it.draw(g);
        }

        painting = false;
    }
    
    
    /**
     *  CONTROL Section
     */
    
    /**
     * respond to custom button press
     * @param code the number of the button pressed (0 ... (Labels.length-1) )
     */
    public void doButton(int code) {
        
        switch(code) {
            case 0:  //  Reset
                System.out.println("Reset");
                reset();
                break;
            case 1:  //  Button A
                System.out.println("Wolf");
                list.add(new Wolf());
                break;
        }
    }
    
    /**
     * Mouse clicked
     * @param e 
     */
    public void clicked(MouseEvent e) {
        System.out.println("Mouse Clicked");
    };
    /**
     * Mouse entered
     * @param e 
     */
    public void entered(MouseEvent e) {
//        System.out.println("Mouse Entered");
    };
    /**
     * Mouse exited
     * @param e 
     */
    public void exited(MouseEvent e) {
//        System.out.println("Mouse Exited");
    };
    /**
     * Mouse released
     * @param e 
     */
    public void released(MouseEvent e) {
        System.out.println("Mouse Released");
    };
    /**
     * Mouse pressed
     * @param e 
     */
    public void pressed(MouseEvent e) {
        System.out.println("Mouse Pressed");
    };
    /**
     * Mouse dragged
     * @param e 
     */
    public void dragged(MouseEvent e) {
//        System.out.println("Mouse Dragged");
    };
    /**
     * Mouse moved
     * @param e 
     */
    public void moved(MouseEvent e) {
//        System.out.println("Mouse Moved");
    };
}
