/**
 * A class providing the storage for a basic panel
 * @author David M. Smith
 */

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.event.*;
import javax.swing.JPanel;

/**
 * display panel for the OTSP solution
 */
public class MyPanel extends JPanel implements Runnable, MouseListener, MouseMotionListener {
    /** the screen size */
     private Dimension screenSize;
    /** whether the solution is running */
     public boolean isRunning;
    /** the solution */
     private Model myModel;

//------------------------------------------------------------------
//
/**
     *   basic constructor
     * @param c the solution
     */
    public MyPanel( Model c ) {
      	myModel = c;
        isRunning = false;
        addMouseListener(this);
        addMouseMotionListener(this);
    }

/**
 *   stop the animation.
*/
    public void stop() {
		isRunning = false; 
    }


/**
 *   run the animation.
*/
    public void run() {
        do {
		myModel.timerTick();
                repaint();
                int dt = (int)(1000 * Global.DT);
		try{ Thread.sleep(dt); } // millisecs
		catch (InterruptedException e) {}
       } while (isRunning);
    }

    
/**
 *   repaint the display
     * blank fill the screen then paint it
     * @param g the graphic environment
     */
    public void paintComponent(Graphics g) {

       Dimension d = getSize();
       myModel.doPaint(g, d);
    }  
    
    
    public void mouseClicked(MouseEvent e) {myModel.clicked(e);};
    public void mouseEntered(MouseEvent e) {myModel.entered(e);};
    public void mouseExited(MouseEvent e) {myModel.exited(e);};
    public void mouseReleased(MouseEvent e) {myModel.released(e);};
    public void mousePressed(MouseEvent e) {myModel.pressed(e);};
    public void mouseDragged(MouseEvent e) {myModel.dragged(e);};
    public void mouseMoved(MouseEvent e) {myModel.moved(e);};
 }


